#include <cstdio>
#include <cstdint>

#include <unistd.h>

#include "../common/CSerial.h"
#include "../common/CSocket.h"
#include "../common/LE9xxCommand.h"

#define AI_NUM 5

CComm *CreateIO()
{
	// -------------------------------------------------
	// Select the communication method with LE-910R.
	// For the USB interface, open the virtual COM port.
	// For the WiFi interfaces, open a TCP socket.
	// -------------------------------------------------
#if 1
	// Open COM port
	CSerial *comm = new CSerial();
	char device[] = "/dev/ttyUSB0";
	if (!comm->Open((void *)device))
	{
		printf("Interface open error.\n");
		delete comm;
		return nullptr;
	}
#else
	// Open TCP socket
	// (IP Address = 192.168.0.110, Control port number = 10101)
	CSocket *comm = new CSocket();
	CSocket::CONNECTION connetion = {"192.168.0.110", 10101};
	if (!comm->Open((void *)&connetion))
	{
		printf("Interface open error.\n");
		delete comm;
		return nullptr;
	}
#endif

	return comm;
}

LE9xxCommand *Connect(CComm *comm)
{
	// ------------------
	// Connect to LE-910R
	// ------------------
	FRAME Res;
	LE9xxCommand *cmd = new LE9xxCommand(comm);

	// Invalid connection maintenance command transmission from LE-910R
	cmd->TransmitCommandWait(CMD_CONNECT, 0x20, 0, NULL, &Res);
	if ((Res.res != RES_OK) && (Res.res != RES_ALREADY_CONNECTED))
	{
		printf("Connection error. [0x%02X]\n", Res.res);
		delete cmd;
		return nullptr;
	}
	printf("Connection OK.\n");

	return cmd;
}

bool Setup(LE9xxCommand *cmd, RANGE (&range_cfg)[AI_NUM])
{
	// -----------------
	// Setup for LE-910R
	// ------------------
	// Thermocouple measurement setting
	// Ch = AI5, Type = K
	// Option = Internal compensation & Enable disconnection detection(output=0x800000)
	FRAME Res;
	uint8_t tc_cfg[3] = {0x10, 0, 0x03};
	cmd->TransmitCommandWait(CMD_TEMP_SET, 0x00, sizeof(tc_cfg), tc_cfg, &Res);
	if (Res.res != RES_OK)
	{
		printf("Thermocouple measurement setting error. [0x%02X]\n", Res.res);
		return false;
	}

	// AD converter conversion speed setting ( SPS=10sps )
	uint8_t sps = 0;
	cmd->TransmitCommandWait(CMD_AD_SPS_SET, 0x00, 1, &sps, &Res);
	if (Res.res != RES_OK)
	{
		printf("AD converter conversion speed setting error. [0x%02X]\n", Res.res);
		return false;
	}

	// Analog input range setting
	for (int i = 0; i < 5; i++)
	{
		uint8_t range[2];
		range[0] = 0x01 << i;	 // Ch mask
		range[1] = range_cfg[i]; // Range
		cmd->TransmitCommandWait(CMD_AD_RANGE_SET, 0x00, sizeof(range), range, &Res);
		if (Res.res != RES_OK)
		{
			printf("Analog input range setting error.(AI%d) [0x%02X]\n", i + 1, Res.res);
			return false;
		}
	}

	return true;
}

void Measure(LE9xxCommand *cmd, RANGE (&range_cfg)[AI_NUM])
{
	// -------------------------------
	// Acquire and convert analog data
	// -------------------------------
	// Acquires and converts analog data of all channels about every second.
	// The measurement is performed 10 times.
	FRAME Res;
	for (int i = 0; i < 10; i++)
	{
		printf("No.%2d", i + 1);
		for (int ch = 0; ch < 5; ch++)
		{
			// Analog data acquisition
			cmd->TransmitCommandWait(CMD_AD_VALUE_GET, 0x00, 1, (uint8_t *)&ch, &Res);
			if (Res.res != RES_OK)
			{
				printf("\nAnalog data acquisition error.(AI%d) [0x%02X]\n", i + 1, Res.res);
				break;
			}

			// Conversion of analog data
			uint32_t ad;
			ad = (uint32_t)Res.data[2] << 16;
			ad |= (uint32_t)Res.data[3] << 8;
			ad |= (uint32_t)Res.data[4];
			float val = cmd->ConvertAnalogDataFromLE910(range_cfg[ch], ad);

			printf(",AI%d:", ch + 1);
			switch (Res.data[1])
			{
			case RANGE_100mV:
				printf("%.3f[mV]", val);
				break;
			case RANGE_1V:
				printf("%.5f[V]", val);
				break;
			case RANGE_10V:
			case RANGE_30V:
				printf("%.4f[V]", val);
				break;
			case RANGE_I_250ohm:
			case RANGE_I_50ohm:
				printf("%.4f[mA]", val);
				break;
			case RANGE_TC:
				if ((val == 0x800000) || (val == 0x7FFFFF))
					printf("BO");
				else
					printf("%.1f[degC]", val);
				break;
			}
		}
		printf("\n");
		sleep(1);
	}
}

bool Disconnect(LE9xxCommand *cmd)
{
	// ----------------------
	// Disonnect from LE-910R
	// ----------------------
	FRAME Res;
	cmd->TransmitCommandWait(CMD_DISCONNECT, 0x00, 0, NULL, &Res);
	if (Res.res != RES_OK)
	{
		printf("Disconnection error. [0x%02X]\n", Res.res);
		return false;
	}
	printf("Disconnection OK.\n");

	return true;
}

int main()
{
	int ret = -1;

	// AI1 = 100mV, AI2 = 10V, AI3 = 30V, AI4 = 4-20mA, AI5 = Thermocouple
	RANGE range_cfg[AI_NUM] = {RANGE_100mV, RANGE_10V, RANGE_30V, RANGE_I_250ohm, RANGE_TC};

	CComm *comm = CreateIO();
	if (!comm)
	{
		return -1;
	}

	LE9xxCommand *cmd = Connect(comm);
	if (!cmd)
	{
		goto error_comm;
	}

	if (!Setup(cmd, range_cfg))
	{
		goto error_cmd;
	}

	Measure(cmd, range_cfg);

	ret = 0;

error_cmd:
	Disconnect(cmd);

error_comm:
	comm->Close();
	delete cmd;

error_exit:
	delete comm;

	return ret;
}
