#include <cstdio>
#include <cstdint>

#include "../common/CSerial.h"
#include "../common/CSocket.h"
#include "../common/LE9xxCommand.h"

CComm *CreateIO()
{
	// -------------------------------------------------
	// Select the communication method with LE-930R.
	// For the USB interface, open the virtual COM port.
	// For the WiFi interfaces, open a TCP socket.
	// -------------------------------------------------
#if 1
	// Open COM port
	CSerial *comm = new CSerial();
	char device[] = "/dev/ttyUSB0";
	if (!comm->Open((void *)device))
	{
		printf("Interface open error.\n");
		delete comm;
		return nullptr;
	}
#else
	// Open TCP socket
	// (IP Address = 192.168.0.130, Control port number = 10101)
	CSocket *comm = new CSocket();
	CSocket::CONNECTION connetion = {"192.168.0.130", 10101};
	if (!comm->Open((void *)&connetion))
	{
		printf("Interface open error.\n");
		delete comm;
		return nullptr;
	}
#endif

	return comm;
}

LE9xxCommand *Connect(CComm *comm)
{
	// ------------------
	// Connect to LE-930R
	// ------------------
	FRAME Res;
	LE9xxCommand *cmd = new LE9xxCommand(comm);

	// Invalid connection maintenance command transmission from LE-930R
	cmd->TransmitCommandWait(CMD_CONNECT, 0x20, 0, NULL, &Res);
	if ((Res.res != RES_OK) && (Res.res != RES_ALREADY_CONNECTED))
	{
		printf("Connection error. [0x%02X]\n", Res.res);
		delete cmd;
		return nullptr;
	}
	printf("Connection OK.\n");

	return cmd;
}

bool Output(LE9xxCommand *cmd)
{
	// -----------------
	// Setup for LE-930R
	// ------------------
	// Input output type
	char input[64] = "";
	LE930_TYPE type;
	printf("Enter the output type number.\n");
	printf(" %d : 100mV range\n", LE930_TYPE_VOLTAGE_100mV);
	printf(" %d : 10V range\n", LE930_TYPE_VOLTAGE_10V);
	printf(" %d : 4-20mA range (Use internal power supply)\n", LE930_TYPE_CURRENT_INT);
	printf(" %d : 4-20mA range (Use external power supply)\n", LE930_TYPE_CURRENT_EXT);
	printf("> ");
	scanf("%d%*[^\n]", &type);
	if ((type < 0) || (type >= LE930_TYPE_NUM))
	{
		printf("The input number is invalid.\n");
		return false;
	}

	// -------------------
	// Analog value output
	// -------------------
	float min, max, value;
	char *unit;
	switch (type)
	{
	case LE930_TYPE_VOLTAGE_100mV:
		min = -100;
		max = 100;
		unit = "mV";
		break;
	case LE930_TYPE_VOLTAGE_10V:
		min = -10;
		max = 10;
		unit = "V";
		break;
	case LE930_TYPE_CURRENT_INT:
	case LE930_TYPE_CURRENT_EXT:
		min = 0;
		max = 20;
		unit = "mA";
		break;
	}
	printf("\nEnter the output value in the range of %.0f%s to %.0f%s.\n", min, unit, max, unit);
	printf(" (Enter "
		   "999"
		   " to finish.)\n");

	// The input value is output from LE-930R.
	FRAME Res;
	for (;;)
	{
		// Input output value
		printf("> ");
		scanf("%f%*[^\n]", &value);
		if ((value >= min) && (value <= max))
		{
			// Output analog value
			uint32_t ad = cmd->ConvertRealDataLE930(type, value);
			uint8_t vout[3] = {0};
			vout[0] = (uint8_t)type;
			vout[1] = (uint8_t)(ad >> 8);
			vout[2] = (uint8_t)(ad & 0xFF);
			cmd->TransmitCommandWait(CMD_DA_OUT_SET, 0x00, sizeof(vout), vout, &Res);
			if (Res.res != RES_OK)
			{
				printf("Analog output error. [0x%02X]\n", Res.res);
				return false;
			}
		}
		else
		{
			if (value == 999)
				break;
			else
				printf("Input value is invalid.\n");
		}
	}

	return true;
}

bool Disconnect(LE9xxCommand *cmd)
{
	// ----------------------
	// Disonnect from LE-930R
	// ----------------------
	FRAME Res;
	cmd->TransmitCommandWait(CMD_DISCONNECT, 0x00, 0, NULL, &Res);
	if (Res.res != RES_OK)
	{
		printf("Disconnection error. [0x%02X]\n", Res.res);
		return false;
	}
	printf("Disconnection OK.\n");

	return true;
}

int main()
{
	int ret = -1;

	CComm *comm = CreateIO();
	if (!comm)
	{
		return -1;
	}

	LE9xxCommand *cmd = Connect(comm);
	if (!cmd)
	{
		goto error_comm;
	}

	if (!Output(cmd))
	{
		goto error_comm;
	}

	ret = 0;

error_cmd:
	Disconnect(cmd);

error_comm:
	comm->Close();
	delete cmd;

error_exit:
	delete comm;

	return ret;
}
